@@ -19,12 +19,9 @@
 //!     30,
 //!     "supersecret",
 //! );
-//! let time = SystemTime::now()
-//!     .duration_since(SystemTime::UNIX_EPOCH).unwrap()
-//!     .as_secs();
 //! let url = totp.get_url("user@example.com", "my-org.com");
 //! println!("{}", url);
-//! let token = totp.generate(time);
+//! let token = totp.generate_current().unwrap();
 //! println!("{}", token);
 //! ```
 //!
@@ -55,6 +52,7 @@ use core::fmt;
 use {base64, image::Luma, qrcodegen};
 
 use hmac::Mac;
+use std::time::{SystemTime, SystemTimeError, UNIX_EPOCH};
 
 type HmacSha1 = hmac::Hmac<sha1::Sha1>;
 type HmacSha256 = hmac::Hmac<sha2::Sha256>;
@@ -103,6 +101,13 @@ impl Algorithm {
     }
 }
 
+fn system_time() -> Result<u64, SystemTimeError> {
+    let t = SystemTime::now()
+        .duration_since(UNIX_EPOCH)?
+        .as_secs();
+    Ok(t)
+}
+
 /// TOTP holds informations as to how to generate an auth code and validate it. Its [secret](struct.TOTP.html#structfield.secret) field is sensitive data, treat it accordingly
 #[derive(Debug, Clone)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
@@ -157,7 +162,7 @@ impl<T: AsRef<[u8]>> TOTP<T> {
         )
     }
 
-    /// Will generate a token according to the provided timestamp in seconds
+    /// Will generate a token given the provided timestamp in seconds
     pub fn generate(&self, time: u64) -> String {
         let result: &[u8] = &self.sign(time);
         let offset = (result.last().unwrap() & 15) as usize;
@@ -169,7 +174,13 @@ impl<T: AsRef<[u8]>> TOTP<T> {
         )
     }
 
-    /// Will check if token is valid by current time, accounting [skew](struct.TOTP.html#structfield.skew)
+    /// Generate a token from the current system time
+    pub fn generate_current(&self) -> Result<String, SystemTimeError> {
+        let t = system_time()?;
+        Ok(self.generate(t))
+    }
+
+    /// Will check if token is valid given the provided timestamp in seconds, accounting [skew](struct.TOTP.html#structfield.skew)
     pub fn check(&self, token: &str, time: u64) -> bool {
         let basestep = time / self.step - (self.skew as u64);
         for i in 0..self.skew * 2 + 1 {
@@ -182,6 +193,12 @@ impl<T: AsRef<[u8]>> TOTP<T> {
         false
     }
 
+    /// Will check if token is valid by current system time, accounting [skew](struct.TOTP.html#structfield.skew)
+    pub fn check_current(&self, token: &str) -> Result<bool, SystemTimeError> {
+        let t = system_time()?;
+        Ok(self.check(token, t))
+    }
+
     /// Will return the base32 representation of the secret, which might be useful when users want to manually add the secret to their authenticator
     pub fn get_secret_base32(&self) -> String {
         base32::encode(
@@ -203,6 +220,8 @@ impl<T: AsRef<[u8]>> TOTP<T> {
     }
 
     /// Will return a qrcode to automatically add a TOTP as a base64 string. Needs feature `qr` to be enabled!
+    /// Result will be in the form of a string containing a base64-encoded png, which you can embed in HTML without needing
+    /// To store the png as a file.
     ///
     /// # Errors
     ///
@@ -228,20 +247,8 @@ impl<T: AsRef<[u8]>> TOTP<T> {
         // Draw the border
         for x in 0..image_size {
             for y in 0..image_size {
-                if y < 8*4 || y >= image_size - 8*4 {
-                    canvas.put_pixel(
-                        x,
-                        y,
-                        Luma([255]),
-                    );
-                    continue;
-                }
-                if x < 8*4 || x >= image_size - 8*4 {
-                    canvas.put_pixel(
-                        x,
-                        y,
-                        Luma([255]),
-                    );
+                if (y < 8*4 || y >= image_size - 8*4) || (x < 8*4 || x >= image_size - 8*4) {
+                    canvas.put_pixel(x, y, Luma([255]));
                 }
             }
         }
@@ -358,11 +365,20 @@ mod tests {
     }
 
     #[test]
-    fn generates_token() {
+    fn generate_token() {
         let totp = TOTP::new(Algorithm::SHA1, 6, 1, 1, "TestSecret");
         assert_eq!(totp.generate(1000).as_str(), "718996");
     }
 
+    #[test]
+    fn generate_token_current() {
+        let totp = TOTP::new(Algorithm::SHA1, 6, 1, 1, "TestSecret");
+        let time = SystemTime::now()
+            .duration_since(SystemTime::UNIX_EPOCH).unwrap()
+            .as_secs();
+        assert_eq!(totp.generate(time).as_str(), totp.generate_current().unwrap());
+    }
+
     #[test]
     fn generates_token_sha256() {
         let totp = TOTP::new(Algorithm::SHA256, 6, 1, 1, "TestSecret");
@@ -384,6 +400,13 @@ mod tests {
         assert!(!totp.check("714250", 2000));
     }
 
+    #[test]
+    fn checks_token_current() {
+        let totp = TOTP::new(Algorithm::SHA1, 6, 0, 1, "TestSecret");
+        assert!(totp.check_current(&totp.generate_current().unwrap()).unwrap());
+        assert!(!totp.check_current("bogus").unwrap());
+    }
+
     #[test]
     fn checks_token_with_skew() {
         let totp = TOTP::new(Algorithm::SHA1, 6, 1, 1, "TestSecret");
